==> 1.0.0-dev.tar.gz: flutterx_reflection_codegen/lib/src/encoder/utils/declaration_builder.dart <==
import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:flutterx_reflection_codegen/src/encoder/source_encoder.dart';
import 'package:flutterx_reflection_codegen/src/encoder/target_sdk.dart';

class DeclarationBuilder {
  static final Set<String> _declaredVariables = {};
  final Map<String, String> _parameters = {};
  final List<String> _declarations = [];
  final List<String> _typeArgs = [];
  final Object _identifier;
  final String _type;

  DeclarationBuilder(this._type, this._identifier);

  void typeArg({required String value}) => _typeArgs.add(value);

  void parameterTokenFunction(String reference, DartType returnType, Iterable<DartType> parametersType) {
    final typeParameters = findTypeParameters([returnType, ...parametersType]).toSet().toList(growable: false);
    if (typeParameters.isEmpty) return parameter(name: 'token', value: '$kTypeToken<$reference>()');
    final variable = sdk.flutter_type_token.identifier(_identifier);
    parameter(name: 'token', value: '$kTypeToken<$kFunction>($variable)');
    final inner = '$kTypeToken<$reference>($variable, t)';
    final body = encodeTokensUsage('t', typeParameters, inner);
    addDeclaration('$kTypeToken $variable($kList<$kTypeToken> t) => $body;');
  }

  void parameterToken(String reference, [List<TypeParameterElement> typeParameters = const []]) {
    if (typeParameters.isEmpty) return parameter(name: 'token', value: '$kTypeToken<$reference>()');
    final variable = sdk.flutter_type_token.identifier(_identifier);
    parameter(name: 'token', value: '$kTypeToken<$reference>($variable)');
    final inner =
        '$kTypeToken<$reference<${typeParameters.map(TypeParameterElementEncoder.instance.reference).join(', ')}>>($variable, t)';
    final body = encodeTokensUsage('t', typeParameters, inner);
    addDeclaration('$kTypeToken $variable($kList<$kTypeToken> t) => $body;');
  }

  void parameterGetter({required String type, required String name, required String value}) {
    final variable = sdk.flutter_parameter.identifier(type, value);
    parameter(name: name, value: variable);
    if (_declaredVariables.contains(variable)) return;
    _declaredVariables.add(variable);
    addDeclaration('$type $variable() => $value;');
  }

  void parameter({required String name, required String value}) => _parameters[name] = value;

  void addDeclaration(String declaration) => _declarations.add(declaration);

  String build({String? variable}) {
    final parameters = _parameters.entries
        .map((parameter) => parameter.key.isNotEmpty ? '${parameter.key}: ${parameter.value}' : parameter.value);
    final out = StringBuffer();
    if (variable != null) out.write('const $variable = ');
    out.write(_type);
    if (_typeArgs.isNotEmpty) out.write('<${_typeArgs.join(', ')}>');
    out.write('(${parameters.join(', ')})');
    if (variable != null) out.writeln(';');
    _declarations.forEach(out.writeln);
    return out.toString().trimRight();
  }
}

==> 1.0.2-dev.tar.gz: flutterx_reflection_codegen/lib/src/encoder/utils/declaration_builder.dart <==
import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:flutterx_reflection_codegen/src/encoder/source_encoder.dart';
import 'package:flutterx_reflection_codegen/src/encoder/target_sdk.dart';

class DeclarationBuilder {
  static final Set<String> _declaredVariables = {};
  final Map<String, _ParameterDescriptor> _parameters = {};
  final List<String> _declarations = [];
  final List<String> _typeArgs = [];
  final Object _identifier;
  final String _type;

  DeclarationBuilder(this._type, this._identifier);

  void typeArg({required String value}) => _typeArgs.add(value);

  void parameterTokenFunction(String reference, DartType returnType, Iterable<DartType> parametersType) {
    final typeParameters = findTypeParameters([returnType, ...parametersType]).toSet().toList(growable: false);
    if (typeParameters.isEmpty) return parameter(name: 'token', value: '$kTypeToken<$reference>()');
    final variable = sdk.flutter_type_token.identifier(_identifier);
    parameter(name: 'token', value: '$kTypeToken<$kFunction>($variable)');
    final inner = '$kTypeToken<$reference>($variable, t)';
    final body = encodeTokensUsage('t', typeParameters, inner);
    addDeclaration('$kTypeToken $variable($kList<$kTypeToken> t) => $body;');
  }

  void parameterToken(String reference, [List<TypeParameterElement> typeParameters = const []]) {
    if (typeParameters.isEmpty) return parameter(name: 'token', value: '$kTypeToken<$reference>()');
    final variable = sdk.flutter_type_token.identifier(_identifier);
    parameter(name: 'token', value: '$kTypeToken<$reference>($variable)');
    final inner =
        '$kTypeToken<$reference<${typeParameters.map(TypeParameterElementEncoder.instance.reference).join(', ')}>>($variable, t)';
    final body = encodeTokensUsage('t', typeParameters, inner);
    addDeclaration('$kTypeToken $variable($kList<$kTypeToken> t) => $body;');
  }

  void parameterGetter({required String type, required String name, required String value}) {
    final variable = sdk.flutter_parameter.identifier(type, value);
    parameter(name: name, value: variable);
    if (_declaredVariables.contains(variable)) return;
    _declaredVariables.add(variable);
    addDeclaration('$type $variable() => $value;');
  }

  void parameter({required String name, bool named = true, required String value}) =>
      _parameters[name] = _ParameterDescriptor(name: name, named: named, value: value);

  void addDeclaration(String declaration) => _declarations.add(declaration);

  String build({String? variable}) {
    final parameters = _parameters.values
        .map((parameter) => parameter.named ? '${parameter.name}: ${parameter.value}' : parameter.value);
    final out = StringBuffer();
    if (variable != null) out.write('const $variable = ');
    out.write(_type);
    if (_typeArgs.isNotEmpty) out.write('<${_typeArgs.join(', ')}>');
    out.write('(${parameters.join(', ')})');
    if (variable != null) out.writeln(';');
    _declarations.forEach(out.writeln);
    return out.toString().trimRight();
  }
}

class _ParameterDescriptor {
  final String name;
  final bool named;
  final String value;

  const _ParameterDescriptor({required this.name, required this.named, required this.value});
}
